# The LearningOnline Network with CAPA
# automated enrollment course setup handler
#
# $Id: lonmodifycourse.pm,v 1.27 2006/07/18 21:52:01 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
package Apache::lonmodifycourse;

use strict;
use Apache::Constants qw(:common :http);
use Apache::lonnet;
use Apache::loncommon;
use Apache::lonlocal;
use Apache::londropadd;
use LONCAPA::Enrollment;
use lib '/home/httpd/lib/perl';
use LONCAPA;

sub print_course_selection_page {
    my ($r,$tasklongref) = @_;
    my $dom = $env{'request.role.domain'};
    my %lt=&Apache::lonlocal::texthash(
                    'csae' => "Course settings for automated enrollment",
                    'unst' => "Unlike standard LON-CAPA course parameters, such as course description, feedback addresses, and top level map, which are displayed and/or modified using the 'Course Environment Parameters' screen, settings that control automated enrollment based on classlist data available from your institution's student information system are handled differently.  Automated enrollment settings fall into two groups: (a) settings that can be modified by a Course Coordinator using the Automated Enrollment Manager and (b)  settings that may only be modified by a Domain Coordinator via the 'View/Modify Course settings' menu accessed from this page.", 
                    'chcs' => "Choose the course for which you wish to view/modify automated enrollment settings from the select box below and click 'Go' to proceed.",
                    'eaen' =>  "Each entry in the select box contains: <b>course code</b> -- <b>course title</b> ------- <b>course owner</b>, and entries are ordered alphabetically by course code.",
                    'psac' => "Please select a course",
                    'ccrs' => "Choose a course",
                    'gobt' => "Go"
    );
                                                                                      
# Determine the courses
    my %courseIDs = &Apache::lonnet::courseiddump($dom,'.',1,'.','.','.',undef,undef,'.');
    &print_header($r,$tasklongref);
    $r->print(<<ENDBLOCK);
<form action="/adm/modifycourse" method="post" name="cmod">
<h3>$lt{'csae'}</h3>
<p>$lt{'unst'}
</p><p>$lt{'chcs'}
<br/>$lt{'eaen'}
</p><p>
<b>$lt{'ccrs'}:</b>
<select name="course">
 <option value="0">$lt{'psac'}</option>
 <option value="-1">---------------------------------------------------</option>
ENDBLOCK
    my $iter = 0;
    my @codes = ();
    my %courses = (); 
    foreach my $key (sort keys %courseIDs) {
# Get current code
        my $crs;
        my ($description,$currcode,$owner);
        if ($courseIDs{$key} =~ m/^([^:]*):([^:]+)/ ) {
            $currcode = $2;
        } elsif ($key =~ m/^($dom)_(\w+)$/) {
            $crs = $2;
            my %settings = &Apache::lonnet::get('environment',['internal.coursecode'],$dom,$crs);
            if (defined($settings{'internal.coursecode'}) ) {
                $currcode = $settings{'internal.coursecode'};
            }
        }
        if ($currcode eq '') {
            $currcode = "___".$iter;
                $iter ++;
        }
        unless (grep/^$currcode$/,@codes) {
            push @codes,$currcode;
            @{$courses{$currcode}} = ();
        }
        push @{$courses{$currcode}}, $key;
    }
    foreach my $code (sort @codes) { 
        foreach my $item (@{$courses{$code}}) { 
            my $crs;
            my $owner;
            my $ownername;
            my $description;
            my $showcode = '';
            unless ($code =~m/^___\d+$/) {  $showcode = $code; }
            if ($item =~ m/^($dom)_(\w+)$/) {
                $crs = $2;
                if ($courseIDs{$item} =~ /^([^:]*):([^:]*):([^:]*)/) {
                    $description = &unescape($1);
                    $owner = &unescape($3);
                    if (($owner ne '') && ($owner !~ /^([^:]+):([^:]+)$/)) {
                        $owner = $owner.':'.$dom;
                    }
                } elsif ($courseIDs{$item} =~ /^([^:]*):([^:]*)$/) {
                    $description = &unescape($1);
                } else {   
                    $description = &unescape($courseIDs{$item});
                }
# Get course owner
                if ($owner eq '') {
                    my %settings = &Apache::lonnet::get('environment',['internal.courseowner'],$dom,$crs);
                    if ( defined($settings{'internal.courseowner'}) ) {
                        $owner = $settings{'internal.courseowner'};
                    }
                }
                unless ($owner eq '') {
                    my ($owneruname,$ownerdom);
                    if ($owner =~ /^([^:]+):([^:]+)$/) {
                        $owneruname = $1;
                        $ownerdom = $2;             
                    } else {
                        $owneruname = $owner;
                        $ownerdom = $dom;
                    }
                    $ownername = &Apache::loncommon::plainname($owneruname,
                                                               $ownerdom);
                }
                $r->print("<option value=\"$crs\">$showcode -- $description ---------- $ownername</option>");
            }
        }
    }
    $r->print("</select>
&nbsp;
<input type=\"hidden\" name=\"action\" value=\"display\" />
<input type=\"button\" onClick=\"this.form.submit()\" value=\"$lt{'gobt'}\" />
</p>
</form>");
    &print_footer($r);
    return;
}

sub print_course_modification_page {
    my ($r,$tasklongref,$typeref) = @_;
    my %enrollvar = ();
    my $javascript_validations;
    my $course = '';
    my $dom = $env{'request.role.domain'};
    if ( defined($env{'form.course'}) ) {
        $course = $env{'form.course'};
    }
    my $ok_course = 'ok';
    if ( ($course == -1) || ($course == '-2') || ($course eq '') ) {
        $ok_course = 'invalid';
    } else {
        $ok_course = &check_course($dom,$course);
    }

    unless ($ok_course eq 'ok') {
        &print_header($r,$tasklongref,'',\$javascript_validations);
        my $reply = "<br/>".&mt("The LON-CAPA course selected was not a valid course for this domain");
        $r->print($reply);
        &print_footer($r);
        return;
    }

    my @bgcolors=("#eeeeee","#cccccc");
    my $ownertable;
    my %lt=&Apache::lonlocal::texthash(
            'actv' => "Active",
            'inac' => "Inactive",
            'ccor' => "Course Coordinator",
            'noen' => "No end date",
            'ownr' => "Owner",
            'name' => "Name",
            'unme' => "Username:Domain",
            'stus' => "Status",
            'aecs' => "Automated Enrollment Course Settings",
            'cose' => "Course settings for LON-CAPA courses that control automated student enrollment based on classlist data available from your institution's student information system fall into two groups: (a) settings that can be modified by a Course Coordinator using the ",
            'aenm' => "Automated Enrollment Manager",
            'andb' => " and (b) settings that may only be modified by a Domain Coordinator via this page.",
            'caes' => 'Current automated enrollment settings',
            'cour' => "Course settings that control automated enrollment in this LON-CAPA course
are currently:",
            'cset' => "Course setting",
            'valu' => "Current value",
            'nocc' => "There is currently no course owner set for this course. In addition, no active course coordinators from this LON-CAPA domain were found, so you will not be able assign a course owner.  If you wish to assign a course owner, it is recommended that you use the 'User Roles' screen to add a course coordinator with a LON-CAPA account in this domain to the course.",    
            'ccus' => "A course coordinator can use the 'Automated Enrollment Manager' to change
all settings except course code, course owner, and default authentication method for students added to your course (who are also new to the LON-CAPA system at this domain).",
            'mkch' => "Make changes to course settings set by Domain Coordinator",
            'ccod' => "Course Code",
            'ccus' => "The course code is used during automated enrollment to map the internal LON-CAPA course ID for this course to the corresponding course section ID(s) used by the office responsible for providing official class lists for courses at your institution.",
            'cown' => "Course Owner",
            'cous' => "The course owner is used in the retrieval of class lists from your institution's student information system when access to class lists data incorporates validation of instructor status.",
            'tabl' => 'The table below contains a list of active course coordinators in this course, who are from this domain',
            'usrd' => 'Use the radio buttons to select a different course owner.',
            'deam' => "Default Authentication method",
            'deus' => "The default authentication method, and default authentication parameter (domain, initial password or argument) are used when automatic enrollment of students in a course requires addition of new user accounts in your domain, and the class list file contains empty entries for the &lt;authtype&gt; and &lt;autharg&gt; properties for the new student. If you choose 'internally authenticated', and leave the initial password field empty, the automated enrollment process will create a randomized password for each new student account that it adds to your LON-CAPA domain.",
            'gobt' => "Modify settings",
    );

    my %settings = &Apache::lonnet::dump('environment',$dom,$course);
    $enrollvar{'autharg'} = '';
    $enrollvar{'authtype'} = '';
    foreach my $item (keys %settings) {
	if ($item =~ m/^internal\.(.+)$/) {
	    my $type = $1;

            if ( ($type eq "autoadds") || ($type eq "autodrops") ) {
	        if ($settings{$item} == 1) {
	            $enrollvar{$type} = "ON";
	        } else {
	            $enrollvar{$type} = "OFF";
	        }
            } elsif ( ($type eq "autostart") || ($type eq "autoend") ) {
	        if ( ($type eq "autoend") && ($settings{$item} == 0) ) {
	            $enrollvar{$type} = $lt{'noen'};
	        } else {
	            $enrollvar{$type} = localtime($settings{$item});
	        }
	    } elsif ($type eq "sectionums") {
		$enrollvar{$type} = $settings{$item};
		$enrollvar{$type} =~ s/,/, /g;
            } elsif ($type eq "authtype" 
		     || $type eq "autharg"    || $type eq "coursecode"
		     || $type eq "crosslistings") {
	        $enrollvar{$type} = $settings{$item};
            } elsif ($type eq 'courseowner') {
                if ($settings{$item} =~ /^[^:]+:[^:]+$/) {
                    $enrollvar{$type} = $settings{$item};
                } else {
                    if ($settings{$item} ne '') {
                        $enrollvar{$type} = $settings{$item}.':'.$dom;
                    }
                }
            }
        } elsif ($item =~ m/^default_enrollment_(start|end)_date$/) {
	    my $type = $1;
            if ( ($type eq 'end') && ($settings{$item} == 0) ) {
                $enrollvar{$item} = $lt{'noen'};
            } elsif ( ($type eq 'start') && ($settings{$item} eq '') ) {
                $enrollvar{$item} = 'When enrolled';
            } else {
                $enrollvar{$item} = localtime($settings{$item});
            }
        }
    }

    my @coursepersonnel = &Apache::lonnet::getkeys('nohist_userroles',$dom,$course);
    my @local_ccs = ();
    my %cc_status = ();
    my %pname = ();
    my $active_cc;
    foreach my $person (@coursepersonnel) {
        my ($role,$user) = split(/:/,$person,2);
        $user =~ s/:$//;
        if (($role eq 'cc') && ($user ne ''))  {
            if (!grep(/^\Q$user\E$/,@local_ccs)) {
                my ($ccname,$ccdom) = split(/:/,$user);
                $active_cc = 
                   &Apache::loncommon::check_user_status($ccdom,$ccname,$dom,
                                                         $course,'cc');
                if ($active_cc eq 'active') {
                    push(@local_ccs,$user);
                    $pname{$user} = &Apache::loncommon::plainname($ccname,$ccdom);
                    $cc_status{$user} = $lt{'actv'};
                }
            }
        }
    }
    if ( (!grep(/^$enrollvar{'courseowner'}$/,@local_ccs)) && 
             ($enrollvar{'courseowner'} ne '') )  {
        my ($owneruname,$ownerdom) = split(/:/,$enrollvar{'courseowner'});
        push(@local_ccs,$enrollvar{'courseowner'});
        $pname{$enrollvar{'courseowner'}} = 
                         &Apache::loncommon::plainname($owneruname,$ownerdom);
        $active_cc = &Apache::loncommon::check_user_status($ownerdom,$owneruname,
                                                           $dom,$course,'cc');
        if ($active_cc eq 'active') {
            $cc_status{$enrollvar{'courseowner'}} = $lt{'actv'};
        } else {
            $cc_status{$enrollvar{'courseowner'}} = $lt{'inac'};
        }
    }
    my $numlocalcc = @local_ccs;

    my $helplink=&Apache::loncommon::help_open_topic('Modify_Course',&mt("Help on Modifying Courses"));
    my ($krbdef,$krbdefdom)=&Apache::loncommon::get_kerberos_defaults($dom);
    my $curr_authtype = '';
    my $curr_authfield = '';
    if ($enrollvar{'authtype'} =~ /^krb/) {
        $curr_authtype = 'krb';
    } elsif ($enrollvar{'authtype'} eq 'internal' ) {
        $curr_authtype = 'int';
    } elsif ($enrollvar{'authtype'} eq 'localauth' ) {
        $curr_authtype = 'loc';
    }
    unless ($curr_authtype eq '') {
        $curr_authfield = $curr_authtype.'arg';
    } 
    $javascript_validations=&Apache::londropadd::javascript_validations('modifycourse',$krbdefdom,$curr_authtype,$curr_authfield);
    my %param = ( formname => 'document.cmod',
	   kerb_def_dom => $krbdefdom,
	   kerb_def_auth => $krbdef,
           mode => 'modifycourse',
           curr_authtype => $curr_authtype,
           curr_autharg => $enrollvar{'autharg'} 
	);
    my $krbform = &Apache::loncommon::authform_kerberos(%param);
    my $intform = &Apache::loncommon::authform_internal(%param);
    my $locform = &Apache::loncommon::authform_local(%param);

    my $disp_table = qq|<table border="0" cellpadding="0" cellspacing="0">
                <tr>
                 <td width="100%" bgcolor="#000000">
                  <table width="100%" border="0" cellpadding="1" cellspacing="0">
                   <tr>
                    <td width="100%" bgcolor="#000000">
                     <table border="0" cellpadding="3" cellspacing="3" bgcolor="#FFFFFF">
                      <tr bgcolor="#CCCC99" align="center">
	               <td><b>$lt{'cset'}</b></td>
	               <td><b>$lt{'valu'}</b></td>
	              </tr>
    |;
    my $iter = 0;
    foreach my $key (sort keys %enrollvar) {
        my $colflag = $iter%2;
        $disp_table .= "<tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
		 <td>$$typeref{$key}</td>
		 <td>$enrollvar{$key}</td>
		</tr>";
        $iter ++;
    }
    $disp_table .= "</table>
                   </td>
                  </tr>
                 </table>
                </td>
               </tr>
              </table>";

    if ($numlocalcc == 0) {
        $ownertable = $lt{'nocc'};
    }

    if ($numlocalcc > 0) {
        @local_ccs = sort @local_ccs;
        $ownertable = qq(
            <input type="hidden" name="numlocalcc" value="$numlocalcc" />
            <table>
             <tr>
               <td>$lt{'tabl'} ($dom). $lt{'usrd'}
              </td>
             </tr>
             <tr><td>&nbsp;</td></tr>
             <tr>
              <td>
               <table border="0" cellpadding="0" cellspacing="0">
                <tr>
                 <td width="100%" bgcolor="#000000">
                  <table width="100%" border="0" cellpadding="1" cellspacing="0">
                   <tr>
                    <td width="100%" bgcolor="#000000">
                     <table border="0" cellpadding="3" cellspacing="3" bgcolor="#FFFFFF">
                      <tr bgcolor="#CCCC99" align="center">
                       <td><b>$lt{'ownr'}</b></td>
                       <td><b>$lt{'name'}</b></td>
                       <td><b>$lt{'unme'}</b></td>
                       <td><b>$lt{'stus'}</b></td>
                      </tr>
        );
        for (my $i=0; $i<@local_ccs; $i++) {
            my $colflag = $i%2;
            $ownertable .= "<tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">";
            if ($local_ccs[$i] eq $enrollvar{'courseowner'}) {
                  $ownertable .= "<td><input type=\"radio\" name=\"courseowner\" value=\"$local_ccs[$i]\" checked=\"checked\" /></td>";
            } else {
                $ownertable .= "<td><input type=\"radio\" name=\"courseowner\" value=\"$local_ccs[$i]\" /></td>";
            }
            $ownertable .= "
                 <td>$pname{$local_ccs[$i]}</td>
                 <td>$local_ccs[$i]</td>
                 <td>$cc_status{$local_ccs[$i]} $lt{'ccor'}</td></tr>";
        }
        $ownertable .= "</table>
                    </td>
                   </tr>
                  </table>
                 </td>
                </tr>
               </table>
              </td>
             </tr>
            </table>";
    }
    &print_header($r,$tasklongref,$settings{'description'},\$javascript_validations);
    $r->print(<<ENDDOCUMENT);
<form action="/adm/modifycourse" method="post" name="cmod">
<h3>$lt{'aecs'}</h3>
$lt{'cose'}<a href="/adm/populate">$lt{'aenm'}</a>$lt{'andb'}
<p>
<table width="100%" cellspacing="0" cellpadding="0">
 <tr>
  <td bgcolor="#CCCC99">&nbsp;</td>
  <td align="left" bgcolor="#CCCC99"><h3>$lt{'caes'}</h3></td>
 </tr>
</table>
<p>
$lt{'cour'}
</p><p>
$disp_table
<br/<br/>
</p><p>

<table width="100%" cellspacing="0" cellpadding="0">
 <tr>
  <td bgcolor="#CCCC99">&nbsp;</td>
  <td align="left" bgcolor="#CCCC99">
   <h3>$lt{'mkch'}</h3>
  </td>
 </tr>
</table>
</p><p>
<table width="100%" cellspacing="6" cellpadding="6">
 <tr>
  <td colspan="2">Use the appropriate text boxes and radio buttons below to change some or all of the four automated enrollment settings that may only be changed by a Domain Coordinator. Click the <b>"$lt{'gobt'}"</b> button to save your changes.</td>
 </tr>
 <tr>
  <td width="50%" valign="top">
   <b>$lt{'ccod'}:</b>&nbsp;&nbsp;
    <input type="text" size="10" name="coursecode" value="$enrollvar{'coursecode'}"/><br/><br/>
    $lt{'ccus'}
  </td>
  <td width="50%" valign="top" rowspan="2">
   <b>$lt{'cown'}:</b><br/><br/>
   $ownertable
   <br/><br/>
   $lt{'cous'}
  </td>
 </tr>
 <tr>
  <td width="50%" valign="top">
   <b>$lt{'deam'}:</b><br/><br/>
   $krbform
   <br/>
   $intform
   <br/>
   $locform
   <br/>
   <br/>
   $lt{'deus'}.
   </td>
   <td width="50%">&nbsp;</td>
 </tr>
</table>
<br/><br/>
<table width="90%" cellpadding="5" cellspacing="0">
 <tr>
  <td align="right">
   <input type="hidden" name="course" value="$course" />
   <input type="hidden" name="action" value="process" />
   <input type="button" onClick="verify_message(this.form)" value="$lt{'gobt'}" />
  </td>
 </tr>
</table>
</form>
<br/>
<br/>
ENDDOCUMENT
    &print_footer($r);
    return;
}

sub modify_course {
    my ($r,$tasklongref,$typeref) = @_;
    my $dom = $env{'user.domain'};
    my $crs = $env{'form.course'};
    unless ( &check_course($dom,$crs) eq 'ok' ) {
        &print_header($r,$tasklongref);
        my $reply = "<br/>".&mt("The LON-CAPA course selected was not a valid course for this domain");
        $r->print($reply);
        &print_footer($r);
        return;
    }

    my %settings = &Apache::lonnet::get('environment',['internal.courseowner','internal.coursecode','internal.authtype','internal.autharg','internal.sectionnums','internal.crosslistings','description'],$dom,$crs);
    my %currattr = ();
    my %newattr = ();
    my %cenv = ();
    my $response;
    my $chgresponse;
    my $nochgresponse;
    my $warning;
    my $reply;
    my @changes = ();
    my @nochanges = ();
    my @sections = ();
    my @xlists = ();
    my $changecode = 0;
    my $changeowner = 0;
    unless ($settings{'internal.sectionnums'} eq'') {
        if ($settings{'internal.sectionnums'} =~ m/,/) {
            @sections = split/,/,$settings{'internal.sectionnums'};
        } else {
            $sections[0] = $settings{'internal.sectionnums'};
        }
    }
    unless ($settings{'internal.crosslistings'} eq'') {
        if ($settings{'internal.crosslistings'} =~ m/,/) {
            @xlists = split/,/,$settings{'internal.crosslistings'};
        } else {
            $xlists[0] = $settings{'internal.crosslistings'};
        }
    }

    my @params = ('courseowner','coursecode','authtype','autharg');
    foreach (@params) {
        my $attr = 'internal.'.$_;
        $currattr{$_} = $settings{$attr};
    }

    my $description = $settings{'description'};
    my %cenv = ();

    if ($env{'form.login'} eq 'krb') {
        $newattr{'authtype'} = $env{'form.login'};
        $newattr{'authtype'} .= $env{'form.krbver'};
        $newattr{'autharg'} = $env{'form.krbarg'};
    } elsif ($env{'form.login'} eq 'int') {
        $newattr{'authtype'} ='internal';
        if ((defined($env{'form.intarg'})) && ($env{'form.intarg'})) {
            $newattr{'autharg'} = $env{'form.intarg'};
        }
    } elsif ($env{'form.login'} eq 'loc') {
        $newattr{'authtype'} = 'localauth';
        if ((defined($env{'form.locarg'})) && ($env{'form.locarg'})) {
            $newattr{'autharg'} = $env{'form.locarg'};
        }
    }
    if ( $newattr{'authtype'}=~ /^krb/) {
        if ($newattr{'autharg'}  eq '') {
            $warning = qq(<font color="red" size="+1">).
	    &mt("As you did not include the default Kerberos domain to be used for authentication in this class, the institutional data used by the automated enrollment process must include the Kerberos domain for each new student").qq(</font></p>);
        }
    }

    if ( exists($env{'form.courseowner'}) ) {
        $newattr{'courseowner'}=$env{'form.courseowner'};
        unless ( $newattr{'courseowner'} eq $currattr{'courseowner'} ) {
            $changeowner = 1;
        } 
    }
													      
    if ( exists($env{'form.coursecode'}) ) {
        $newattr{'coursecode'}=$env{'form.coursecode'};
        unless ( $newattr{'coursecode'} eq $currattr{'coursecode'} ) {
            $changecode = 1;
        }
    }
    if ($changeowner == 1 || $changecode == 1) {
        my $courseid_entry = &escape($dom.'_'.$crs).'='.&escape($description).':'.&escape($env{'form.coursecode'}).':'.&escape($env{'form.courseowner'});
        &Apache::lonnet::courseidput($dom,$courseid_entry,&Apache::lonnet::homeserver($crs,$dom));
    }

    foreach (@params) {
        if ($currattr{$_} eq $newattr{$_}) {
            push @nochanges, $_;
        } else {
            my $attr = 'internal.'.$_;
            $cenv{$attr} = $newattr{$_};
            push @changes, $_;
        }
    }

    if (@changes > 0) {
        $chgresponse = &mt("The following automated enrollment parameters have been changed:<br/><ul>");
    }
    if (@nochanges > 0) { 
        $nochgresponse = &mt("The following automated enrollment parameters remain unchanged:<br/><ul>");
    }
    if (@changes > 0) { 
        my $putreply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
        if ($putreply !~ /^ok$/) {
            $response = &mt("There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>");
        } else {
            foreach my $attr (@params) {
                if (grep/^$attr$/,@changes) {
	            $chgresponse .= "<li>$$typeref{$attr} ".&mt("now set to \"").$newattr{$attr}."\".</li>";
                } else {
	            $nochgresponse .= "<li>$$typeref{$attr} ".&mt("still set to \"").$currattr{$attr}."\".</li>";
                }
            }
            if ($changecode || $changeowner) {
                if ( $newattr{'courseowner'} eq '') {
	            $warning .= &mt("There is no owner associated with this LON-CAPA course.  If automated enrollment in LON-CAPA courses at your institution requires validation of course owners, automated enrollment will fail for this course.<br/>");
                } else {
	            if (@sections > 0) {
                        if ($changecode) {
	                    foreach my $sec (@sections) {
		                if ($sec =~ m/^(.+):/) {
		                    my $inst_course_id = $newattr{'coursecode'}.$1;
                                    my $course_check = &Apache::lonnet::auto_validate_courseID($crs,$dom,$inst_course_id);
			            if ($course_check eq 'ok') {
                                        my $outcome = &Apache::lonnet::auto_new_course($crs,$dom,$inst_course_id,$newattr{'courseowner'});
			                unless ($outcome eq 'ok') { 
				            $warning .= &mt("If automatic enrollment is enabled for LON-CAPA course: ").$description.&mt(", automated enrollment may fail for ").$newattr{'coursecode'}.&mt(" - section $1 for the following reason: $outcome.<br/>");
			                }
			            } else {
			                $warning .= &mt("If automatic enrollment is enabled for LON-CAPA course: ").$description.&mt(", automated enrollment may fail for ").$newattr{'coursecode'}.&mt(" - section $1 for the following reason: $course_check.<br/>");
			            }
		                } else {
			            $warning .= &mt("If automatic enrollment is enabled for LON-CAPA course: ").$description.&mt(", automated enrollment may fail for ").$newattr{'coursecode'}.&mt(" - section $sec because this is not a valid section entry.<br/>");
		                }
		            }
	                } elsif ($changeowner) {
                            foreach my $sec (@sections) {
                                if ($sec =~ m/^(.+):/) {
                                    my $inst_course_id = $newattr{'coursecode'}.$1;
                                    my $outcome = &Apache::lonnet::auto_new_course($crs,$dom,$inst_course_id,$newattr{'courseowner'});
                                    unless ($outcome eq 'ok') {
                                        $warning .= &mt("If automatic enrollment is enabled for LON-CAPA course: ").$description.&mt(", automated enrollment may fail for ").$newattr{'coursecode'}.&mt(" - section $1 for the following reason: $outcome.<br/>");
                                    }
                                } else {
                                    $warning .= &mt("If automatic enrollment is enabled for LON-CAPA course: ").$description.&mt(", automated enrollment may fail for ").$newattr{'coursecode'}.&mt(" - section $sec because this is not a valid section entry.<br/>");
                                }
                            }
                        }
	            } else {
	                $warning .= &mt("As no section numbers are currently listed for LON-CAPA course: ").$description.&mt(", automated enrollment will not occur for any sections of coursecode: ").$newattr{'coursecode'}."<br/>";
	            }
	            if ( (@xlists > 0) && ($changeowner) ) {
	                foreach my $xlist (@xlists) {
		            if ($xlist =~ m/^(.+):/) {
                                my $outcome = &Apache::lonnet::auto_new_course($crs,$dom,$1,$newattr{'courseowner'});
		                unless ($outcome eq 'ok') {
			            $warning .= &mt("If automatic enrollment is enabled for LON-CAPA course: ").$description.&mt(", automated enrollment may fail for crosslisted class: ").$1.&mt(" for the following reason: $outcome.<br/>");
		                }
		            } 
	                }
	            }
                }
            }
        }
    } else {
        foreach my $attr (@params) {
            $nochgresponse .= "<li>$$typeref{$attr} ".&mt("still set to")." \"".$currattr{$attr}."\".</li>";
        }
    }


    if (@changes > 0) {
        $chgresponse .= "</ul><br/><br/>";
    }
    if (@nochanges > 0) {
        $nochgresponse .=  "</ul><br/><br/>";
    }
    unless ($warning eq '') {
        $warning = &mt("The following warning messages were generated as a result of applying the changes you specified to course settings that can affect the automated enrollment process:<br/><br/>").$warning;
    }
    if ($response eq '') {
        $reply = $chgresponse.$nochgresponse.$warning;
    } else {
        $reply = $response;
    }
    &print_header($r,$tasklongref,$description);
    $reply = "<h3>".&mt("Automated Enrollment Course Settings")."</h3><table><tr><td>".$reply."</td></tr></table>";
    $r->print($reply);
    &print_footer($r);
    return; 
}

sub print_header {
    my ($r,$tasklongref,$description,$javascriptref) = @_;
    my %lt =&Apache::lonlocal::texthash(
             'chco' => 'Choose a course',
             'main' => 'Main Menu',
             'comg' => 'Course Manager',
             );
    my $action = "start";
    if ( exists($env{'form.action'}) ) {
        $action = $env{'form.action'};
    }
    if ( ($description eq '') || (!defined($description)) ) {
        $description = $lt{'comg'};
    }
    my $page = '';

    if ($action eq 'start') {  
        $page = "<b>$lt{'chco'}</b>";
    } else {
        $page =  '<a href="/adm/modifycourse">'.$lt{'chco'}.'</a>';
        if ( $action eq 'process' ) {
            my $course = $env{'form.course'};
            $page .= "-&gt; <a href=\"/adm/modifycourse?action=display&course=$course\">".$$tasklongref{'display'}."</a> -&gt; <b>$$tasklongref{$action}</b> ";
        } else {
            $page .=  " -&gt; <b>".$$tasklongref{$action}."</b>";
        }
    }

    my $js;
    if ($action eq 'display') {
	$js = '<script  type="text/javascript">
                  '.$$javascriptref.'
               </script>';
    }
    $r->print(&Apache::loncommon::start_page('View/Modify Course Settings',
					     $js));
    $r->print(<<ENDTHIS);
<table width="100%" border="0" cellpadding="0" cellspacing="0">
 <tr>
  <td bgcolor="#CCCC99">
   <font size="2"><a href="/adm/menu">$lt{'main'}</a> -&gt; $page</font><br/>
  </td>
  <td align="right" bgcolor="#CCCC99" valign="top">
   <font size="+1">$description &nbsp;</font>
  </td>
 </tr>
</table>
ENDTHIS
    return;
}

sub print_footer {
    my ($r) = @_;
    $r->print('<br />'.&Apache::loncommon::end_page());
    return;
}

sub check_course {
    my ($dom,$course) = @_;
    my %courseIDs = &Apache::lonnet::courseiddump($dom,'.',1,'.','.','.',undef,undef,'.');
    foreach my $key (sort keys %courseIDs) {
        if ($key =~ m/^($dom)_(\w+)$/) {
            if ($2 eq $course) {
                return 'ok';
            }
        }
    }
    return 'invalid course';
}


sub handler {
    my $r = shift;
    if ($r->header_only) {
        &Apache::loncommon::content_type($r,'text/html');
        $r->send_http_header;
        return OK;
    }
                                                                                  
    if (&Apache::lonnet::allowed('ccc',$env{'request.role.domain'})) {
        &Apache::loncommon::content_type($r,'text/html');
        $r->send_http_header;

        &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['action','course']);
        my $dom = $env{'user.domain'};
        my %longtype=&Apache::lonlocal::texthash(
                       'authtype' => 'Default authentication method',
                       'autharg'  => 'Default authentication parameter',
                       'autoadds' => 'Automated adds',
                       'autodrops' => 'Automated drops',
                       'autostart' => 'Date of first automated enrollment',
                       'autoend' => 'Date of last automated enrollment',
                       'default_enrollment_start_date' => 'Date of first student access',
                       'default_enrollment_end_date' => 'Date of last student access',
                       'coursecode' => 'Official course code',
                       'courseowner' => "Username:domain of course owner",
                       'notifylist' => 'Course Coordinators to be notified of enrollment changes',
                       'sectionnums' => 'Course section number(:groupID)',
                       'crosslistings' => 'Crosslisted class(:groupID)',
                      );

        my %tasklong = &Apache::lonlocal::texthash(
                        'display' => 'View/modify settings',
                        'process'  => 'Results of changes',
                       );
                                                                                  
        if ($env{'form.action'} eq 'process') {
            &modify_course($r,\%tasklong,\%longtype);
        } elsif ($env{'form.action'} eq 'display')  {
            &print_course_modification_page($r,\%tasklong,\%longtype);
        } else {
            &print_course_selection_page($r,\%tasklong);
        }
    } else {
        $env{'user.error.msg'}=
        "/adm/modifycourse:ccc:0:0:Cannot modify course settings";
        return HTTP_NOT_ACCEPTABLE;
    }
    return OK;
}

1;
__END__
